#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: SkySearch_plugin.tcl,v 2.43 2004/06/17 23:04:30 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# A.Brighton 10 Nov 97   created
# mkiekebu 2004-06-14    Fixed problem with dialog windows (RH9.0)
#
#************************************************************************
#   NAME
#       SkySearch_plugin - SkyCatTool plugin for astrocat 
#
#   SYNOPSIS
#
#   DESCRIPTION
#        This is an example plugin file for the skycat catalog windows.
#        It can be used to add features to the catalog windows at startup.
#        To use this file, copy it to $HOME/.astrocatrc or set the environment
#        variable ASTOCATRC to the path name of the file.
#
#        This file must define at least one tcl proc: astrocat_plugin, which
#        takes one argument: the name of the top level SkySearch widget.
#        You can refer to the man pages and source code for the Skycat and
#        RTD Itcl widgets for details.
#
#        This file offers some examples and comments that should make it
#        easy to make a simple plugin.
#
#        FORS particularities:
#        Interaction with a user-specified astrometric catalog
#
#   FILES
#
#   COMMANDS
#       new_plot_proc {this} 
#                   example 
#       remove_FORS {this info}
#                   removes a user-defined sublist of the catalog 
#                   from the catalog list
#       add_fors_menubutton {w}
#                  adds a 'VLT-FORS' menu button to the 
#                  catalog widget menu bar          
#       add_prio_col {w}
#                  adds a column with header 'prio' in case there
#                  is not yet such a column in the local catalog
#                  This column will be used later by AutoPos as a
#                  selection criteria
#       add_mag_col {w}
#                  adds a column with header 'mag' in case there
#                  is not yet such a column in the local catalog
#                  This column will be used later by AutoPos as a
#                  selection criteria
#                  
#       
#   ENVIRONMENT
#        HP-UX-10.20, ESO SkyCat Tool
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------
#
# astrocatrc - example SkySearch startup file for plug-ins
#
# This is an example plugin file for the skycat catalog windows. 
# It can be used to add features to the catalog windows at startup. 
# To use this file, copy it to $HOME/.astrocatrc or set the environment 
# variable ASTOCATRC to the path name of the file. 
#
# This file must define at least one tcl proc: astrocat_plugin, which
# takes one argument: the name of the top level SkySearch widget. 
# You can refer to the man pages and source code for the Skycat and
# RTD Itcl widgets for details.
#
# This file offers some examples and comments that should make it
# easy to make a simple plugin. 
#
# who         when       what


# This proc is required. It will be called once for each SkySearch
# instance. The parameter is the name ("$this") of the SkySearch
# class object.

#----------------------------
proc SkySearch_plugin {this} {
#----------------------------
    global astrocat_plugin_initialized_
    # puts stdout "SkySearch_plugin scanned"

    # get the toplevel widget name
    set w [utilNamespaceTail $this]

    # add a "Priority" menu button
    add_fors_menubutton $w

    # add a special method to remove targets from the 
    # astrocat list by FORS
    # set dummy ""
    # remove_FORS  $w $dummy

    # this code should only run once
    if {! [info exists astrocat_plugin_initialized_]} {

	# this is one way to override a class method without subclassing

	# if you want to keep the original method body and add to it, 
	# you could get the body of the method using the itcl info
	# command, insert your code, and then write it back again:
	       # set body [lindex [$this info function plot] 4]
	       # body SkySearch::plot {} "new_plot_proc \$this \n$body"

    }
    set astrocat_plugin_initialized_ 1
}


# replacement for SkySearch plot method

#--------------------------
proc new_plot_proc {this} {
#--------------------------
    info_dialog "plot method overridden for [$this cget -catalog]" .skycat1
}


# remove the currently FORS targets
# copy of remove_selected 
# takes input $info as command line parameter
#----------------------------------
proc remove_FORS {this info} {
#----------------------------------
    #set file $itk_option(-catalog)
    # does not work, since itk_option not available for procs
    set file [$this cget -catalog]

    # puts stdout "RF: this = $this"
    # puts stdout "RF: info = $info"
    # puts stdout "RF: file = $file"

    if {[llength $info] == 0} {
        # error_dialog "No rows are selected" $this
        return;
    }
    if {! [confirm_dialog "Remove object with id \
             [lindex [lindex $info 0] 0]" $this]} {
        return
    }
    
    puts stdout "after confirm_dialog"
    puts stdout "$this.cat"
    puts stdout [$this.cat iswcs]

    if {[$this.cat iswcs]} {
        puts stdout "iswcs old equinox = $equinox"
        set equinox [$equinox_ get]
        puts stdout "iswcs new equinox = $equinox"
    } else {
        puts stdout "old equinox = $equinox"
        set equinox 2000
        puts stdout "new equinox = $equinox"
    }

    puts stdout "before remove "
    if {[catch {$this remove $file $info $equinox $headings_} msg]} {
        error_dialog $msg $this 
        return
    }
    puts stdout "after  remove "
    # update the display
    search
}

# add a menu button to the menubar with some example menu items.
# $w is the name of the TopLevelWidget containing the menubar. 

#-----------------------------
proc add_fors_menubutton {w} {
#-----------------------------
    set m [$w add_menubutton "VLT-FoRS-1" ]

    $w add_menuitem $m command "Add Priority Column" \
	{Adding a new column} \
	-command [code add_prio_col $w]
    
    $w add_menuitem $m command "Add Magnitude Column" \
	{Adding a new column} \
	-command [code add_mag_col $w]
    
    $m add separator

    $w add_menuitem $m command "Add Columns for WCS align" \
	{Adding a new column} -state disabled \
	-command [code add_fitxy_col $w]
    
#   $w add_menuitem $m checkbutton "Test checkbutton" \
#       {Test adding a checkbutton menu item} \
#      -variable test -onvalue 1 -offvalue 0 \
#       -command [code add_prio_col $w]

    $w add_short_help $w.menubar.vlt-fors-1 \
	"Test menu: added in .skycatrc"
}


# dummy callback for menu items
#-----------------------
proc add_prio_col {w} {
#-----------------------
    # if there is no 'prio' column, add one 
    set loc_headings [$w info variable headings_]
    set loc_headings [lindex $loc_headings 4]

    if { [string match "* prio *" $loc_headings] } {
        info_dialog "There is already \n a column with 'prio'" $w
        } else {
        append loc_headings " prio"
        set loc_info [$w info variable info_]
        set loc_info [lindex $loc_info 4]
        set loc_length [llength $loc_info]

        for {set i 0} { $i< $loc_length} { incr i 1} {
            set star [lindex $loc_info $i]
            lappend star "0"; # default low priority
            set loc_info [lreplace $loc_info $i $i $star] 
            }
        $w query_done "" $loc_headings $loc_info 0
       }
}


#-----------------------
proc add_mag_col {w} {
#-----------------------
    # if there is no 'mag' column, add one
    set loc_headings [$w info variable headings_]
    set loc_headings [lindex $loc_headings 4]

    if { [string match "* mag *" $loc_headings] } {
        info_dialog "There is already \n a column with 'mag'" $w
        } else {
        append loc_headings " mag"
        set loc_info [$w info variable info_]
        set loc_info [lindex $loc_info 4]
        set loc_length [llength $loc_info]

        for {set i 0} { $i< $loc_length} { incr i 1} {
            set star [lindex $loc_info $i]
            lappend star "10.0"; # default mag 10.m0
            set loc_info [lreplace $loc_info $i $i $star]
            }
        $w query_done "" $loc_headings $loc_info 0
       }
}

#-----------------------
proc add_fitxy_col {w} {
#-----------------------
# for the particular table to align WCS--PIXEL 
# we need 4 further columns 
     add_x_col $w 
     add_y_col $w
     add_DELTA_RA_col $w
     add_DELTA_DEC_col $w
}

#-----------------------
proc add_x_col {w} {
#-----------------------
    # if there is no 'x' column, add one 
    set loc_headings [$w info variable headings_]
    set loc_headings [lindex $loc_headings 4]

    if { [string match "* x *" $loc_headings] } {
        info_dialog "There is already \n a column with 'x'" $w
        } else {
        append loc_headings " x"
        set loc_info [$w info variable info_]
        set loc_info [lindex $loc_info 4]
        set loc_length [llength $loc_info]

        for {set i 0} { $i< $loc_length} { incr i 1} {
            set star [lindex $loc_info $i]
            lappend star "0"; # default low priority
            set loc_info [lreplace $loc_info $i $i $star] 
            }
        $w query_done "" $loc_headings $loc_info 0
       }
}

#-----------------------
proc add_y_col {w} {
#-----------------------
    # if there is no 'y' column, add one 
    set loc_headings [$w info variable headings_]
    set loc_headings [lindex $loc_headings 4]

    if { [string match "* y *" $loc_headings] } {
        info_dialog "There is already \n a column with 'y'" $w
        } else {
        append loc_headings " y"
        set loc_info [$w info variable info_]
        set loc_info [lindex $loc_info 4]
        set loc_length [llength $loc_info]

        for {set i 0} { $i< $loc_length} { incr i 1} {
            set star [lindex $loc_info $i]
            lappend star "0"; # default 
            set loc_info [lreplace $loc_info $i $i $star] 
            }
        $w query_done "" $loc_headings $loc_info 0
       }
}

#-----------------------
proc add_DELTA_RA_col {w} {
#-----------------------
    # if there is no 'DELTA_RA' column, add one 
    set loc_headings [$w info variable headings_]
    set loc_headings [lindex $loc_headings 4]

    if { [string match "* DELTA_RA *" $loc_headings] } {
        info_dialog "There is already \n a column with 'DELTA_RA'" $w
        } else {
        append loc_headings " DELTA_RA"
        set loc_info [$w info variable info_]
        set loc_info [lindex $loc_info 4]
        set loc_length [llength $loc_info]

        for {set i 0} { $i< $loc_length} { incr i 1} {
            set star [lindex $loc_info $i]
            lappend star "0"; # default
            set loc_info [lreplace $loc_info $i $i $star] 
            }
        $w query_done "" $loc_headings $loc_info 0
       }
}


#-----------------------
proc add_DELTA_DEC_col {w} {
#-----------------------
    # if there is no 'DELTA_DEC' column, add one 
    set loc_headings [$w info variable headings_]
    set loc_headings [lindex $loc_headings 4]

    if { [string match "* DELTA_DEC *" $loc_headings] } {
        info_dialog "There is already \n a column with 'DELTA_DEC'" $w
        } else {
        append loc_headings " DELTA_DEC"
        set loc_info [$w info variable info_]
        set loc_info [lindex $loc_info 4]
        set loc_length [llength $loc_info]

        for {set i 0} { $i< $loc_length} { incr i 1} {
            set star [lindex $loc_info $i]
            lappend star "0"; # default 
            set loc_info [lreplace $loc_info $i $i $star] 
            }
        $w query_done "" $loc_headings $loc_info 0
       }
}

#
# ___oOo___

