#! /bin/sh
#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: midas2skycat.sh,v 2.43 2004/06/17 23:04:35 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
#  hummel    08/12/99  created 

#************************************************************************
#   NAME
#       midas2skycat.sh - converts midas tables to skycat catalogues
#
#   SYNOPSIS
#       midas2skycat.sh inputfile outputfile 
#
#   DESCRIPTION
#       This shell script takes the output of 
#       the ESO-MIDAS batch midas2skycat.prg and converts it into a 
#       ESO-SkyCat readable catalog
#       The conversion does via a simple sed script 
#       This newly created catalog in the SkyCat format can be used 
#       in frame of the FIMS MOS/MXU automatic slit positioning.
#       The advantage of this catalog is that the object positions 
#       coincide with the optical positions of the FORS input frame 
#       and should be prefered with respect to any other archive catalog
#       
#
#   FILES
#       midas2skycat.sh, midas2skycat.prg
#
#   ENVIRONMENT
#       UNIX 
#       ESO-MIDAS
#       ESO-SkyCat
#       USM-FIMS 
#
#   RETURN VALUES
#       none
#
#   CAUTIONS
#       The inputfile must be created with the 
#       ESO-MIDAS batch midas2skycat.prg
#
#   EXAMPLES
#       midas2skycat.sh NGC_0815.targ NGC_0815.ptl
#
#   SEE ALSO
#       man page of sed
#       
#   BUGS
#
#------------------------------------------------------------------------
#

#!/usr/bin/sh

sed 's/..........//
s/       / /g
s/      / /g
s/     / /g
s/    / /g
s/   / /g
s/  / /g
s/ //
s/ /	/
s/ /:/
s/ /:/
s/ /	/
s/ /:/
s/ /:/
s/ /	/
s/ /	/
s/ /	/
/^\*/d
1i\
QueryResult\
\
# Config entry for original catalog server:\
serv_type: local\
long_name: result.ptl\
short_name: result.ptl\
url: result.ptl\
symbol: {} {square yellow {} {} {} {}} {2.78e-4 {deg 2000}}:{} {cross red {} {} {} {}} {2.78e-4 {deg 2000}}\
sort_cols: midas_id\
# End config entry\
\
midas_id	ra	dec	bg	pk	ar\
--------	--	---	--	--	--
' $1 > $2 
